#!/usr/bin/env python3
import pytest
import os
from fretraj import burst
import jsonschema

_TEST_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))

BURST_PARAMETERS = {
        "dyes": {
            "tauD": 0.75,
            "tauA": 1.5,
            "QD": 0.2,
            "QA": 0.3
        },
        "sampling": {
            "nbursts": 2000,
            "skipframesatstart": 0,
            "skipframesatend": 1000,
            "multiprocessing": True
        },
        "fret": {
            "R0": 54,
            "kappasquare": 0.666666,
            "no_gamma": True,
            "quenching_radius": 1.0
        },
        "species": {
            "name": ["all"],
            "unix_pattern_rkappa": ["*.dat"],
            "unix_pattern_don_coords": ["*donor*.xvg"],
            "unix_pattern_acc_coords": ["*acceptor*.xvg"],
            "probability": [1]
        },
        "bursts": {
            "lower_limit": 20,
            "upper_limit": 150,
            "lambda": -2.3,
            "QY_correction": False,
            "averaging": "all"
        }
    }


@pytest.fixture
def patch_open(mocker):
    mocker.patch('fretraj.burst.open')
    mocker.patch('json.load', return_value=BURST_PARAMETERS)


@pytest.fixture
def validate_parameters(patch_open):
    return burst.readParameters('anyname.json')


def test_donor_lifetime(validate_parameters):
    assert validate_parameters['dyes']['tauD'] == 0.75


def test_missing_key(validate_parameters, monkeypatch):
    monkeypatch.delitem(BURST_PARAMETERS['dyes'], 'tauD')
    with pytest.raises(jsonschema.exceptions.ValidationError):
        burst.readParameters('anyname.json')


def test_wrong_type(validate_parameters, monkeypatch):
    monkeypatch.setitem(BURST_PARAMETERS['dyes'], 'tauD', '0.75')
    with pytest.raises(jsonschema.exceptions.ValidationError):
        burst.readParameters('anyname.json')


def testTrajectory_rkappa():
    traj = burst.Trajectory.from_file(rkappa_filename=os.path.join(_TEST_DIR, 'data', 'test_rkappa1.dat'))
    assert traj.time[0] == 0


def testTrajectory_rkappa_dyecoords():
    traj = burst.Trajectory.from_file(rkappa_filename=os.path.join(_TEST_DIR, 'data', 'test_rkappa1.dat'),
                                      don_coords_filename=os.path.join(_TEST_DIR, 'data', 'test_donorcoords1.xvg'),
                                      acc_coords_filename=os.path.join(_TEST_DIR, 'data', 'test_acceptorcoords1.xvg'))
    assert traj.time[0] == 0


def testSpecies():
    species = burst.Species(name='all', probability=1,
                            filelist_rkappa=[os.path.join(_TEST_DIR, 'data', 'test_rkappa1.dat'),
                                             os.path.join(_TEST_DIR, 'data', 'test_rkappa2.dat')])
    assert species.name == 'all'
    assert species.trajectories[0].time[0] == 0
    assert pytest.approx(species.trajectories[0].weight, abs=0.01) == 0.4


def testEnsemble():
    ensemble = burst.Ensemble(os.path.join(_TEST_DIR, 'data'),
                              {'species': {'name': ['all'], 'unix_pattern_rkappa': ['*.dat'],
                                           'probability': [1], 'n_trajectory_splits': None}})
    assert len(ensemble.species[0].trajectories) == 2


if __name__ == '__main__':
    pytest.main()



#!/usr/bin/env python3

import pytest
from fretraj import burst
import os
import numpy as np

_TEST_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


@pytest.fixture
def setup_parameters(mocker):
    BURST_PARAMETERS = {
        "dyes": {
            "tauD": 0.75,
            "tauA": 1.5,
            "QD": 0.2,
            "QA": 0.3
        },
        "sampling": {
            "nbursts": 2000,
            "skipframesatstart": 0,
            "skipframesatend": 1000,
            "multiprocessing": True
        },
        "fret": {
            "R0": 54,
            "kappasquare": 0.666666,
            "no_gamma": True,
            "quenching_radius": 1.0
        },
        "species": {
            "name": ["all"],
            "unix_pattern_rkappa": ["*.dat"],
            "unix_pattern_don_coords": ["*donor*.xvg"],
            "unix_pattern_acc_coords": ["*acceptor*.xvg"],
            "probability": [1]
        },
        "bursts": {
            "lower_limit": 20,
            "upper_limit": 150,
            "lambda": -2.3,
            "QY_correction": False,
            "averaging": "all"
        }
    }
    mocker.patch('fretraj.burst.open')
    mocker.patch('json.load', return_value=BURST_PARAMETERS)
    return burst.readParameters('anyname.json')


def test_singleCore_withoutAnisotropy(setup_parameters):
    setup_parameters['sampling']['multiprocessing'] = False
    exp = burst.Experiment(os.path.join(_TEST_DIR, 'data'), setup_parameters,
                           binwidth=0.025, verbose=False, compute_anisotropy=False, units='nm')
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


def test_multiCore_withoutAnisotropy(setup_parameters):
    exp = burst.Experiment(os.path.join(_TEST_DIR, 'data'), setup_parameters,
                           binwidth=0.025, verbose=False, compute_anisotropy=False, units='nm')
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


def test_multiCore_withAnisotropy(setup_parameters):
    exp = burst.Experiment(os.path.join(_TEST_DIR, 'data'), setup_parameters,
                           binwidth=0.025, verbose=False, compute_anisotropy=True, units='nm')
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


if __name__ == "__main__":
    pytest.main()



import sys

try:
    import importlib.metadata as meta  # Python >=3.8
except ModuleNotFoundError:
    import importlib_metadata as meta  # Python 3.7

PACKAGE = 'fretraj'

try:
    metadata = meta.metadata(PACKAGE)
except meta.PackageNotFoundError:
    print(f'{PACKAGE} is not installed yet.')


def _get_urls():
    __urls__ = {}
    try:
        for _url in metadata.get_all('Project-URL'):
            __urls__.update(dict([_url.replace(' ', '').split(',')]))
    except TypeError:
        __urls__['Documentation'] = None
        __urls__['Repository'] = None
    return __urls__


__version__ = metadata['Version']
__author__ = metadata['Author']
__keywords__ = metadata['Keywords']
__license__ = metadata['License']
__urls__ = _get_urls()

from . import cloud
from . import export
from . import fret
from . import grid
from . import jupyter

if sys.platform.startswith('linux'):
    try:
        from . import burst
    except (ImportError, ModuleNotFoundError):
        print('The burst submodule could not be imported\n')
else:
    print('The burst submodule is currently only available on Linux\n')

import warnings
import numba

warnings.simplefilter('ignore', category=numba.errors.NumbaDeprecationWarning)
warnings.simplefilter('ignore', category=numba.errors.NumbaPendingDeprecationWarning)
